﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;
using System.Runtime.Loader;

namespace WinFormsApp1
{
    public partial class Form1 : Form
    {
        /***************************************************************************************************************************
         *                                          基于Winform原生多语言方案的验证
         *                                          1、资源文件可以放在程序资源文件中
         *                                          2、其他界面可以访问别的界面的资源文件
         *                                          3、资源文件最终生成的为可执行dll，无法进行修改
         *                                          4、资源文件中，属性名与控件名称相同
         *                                          5、切换语言时需要枚举所有控件
         **************************************************************************************************************************/

        Form2 form2;
        public Form1()
        {
            InitializeComponent();
            var result = System.IO.Directory.GetFiles(AppDomain.CurrentDomain.BaseDirectory, "Language.*.dll", SearchOption.AllDirectories);
            List<ILanguage> languages = result.SelectMany(x =>
            {
                try
                {
                    System.Reflection.Assembly assembly = System.Reflection.Assembly.LoadFrom(x);
                    if (assembly != null)
                    {
                        return assembly.GetTypes().Where(y => y.GetInterface(typeof(ILanguage).FullName) != null).Select(y => (ILanguage)Activator.CreateInstance(y));
                    }
                    return new List<ILanguage>();
                }
                catch
                {
                    return new List<ILanguage>();
                }
            }).Where(x => x != null).ToList();
            this.comboBox1.DataSource = languages.Select(x =>
            {
                return new KeyValuePair<string, ILanguage>(x.CultureInfo.Name, x);
            }).ToArray();
            this.comboBox1.DisplayMember = "Key";
            this.comboBox1.ValueMember = "Value";
            this.comboBox1.SelectedValueChanged += (sender, args) =>
            {
                LanguageManger.Instance.Language = this.comboBox1.SelectedValue as ILanguage;
            };
            LanguageManger.Instance.LanguageChanged += (sender, args) =>
              {
                  SetLanguge(args);
              };
            LanguageManger.Instance.Language = languages.First();
            this.comboBox1.SelectedValue = languages.First();
        }

        public void SetLanguge(ILanguage language)
        {
            if (form2 != null) form2.SetLanguage(language);

            Controls.Cast<Control>().ToList().ForEach(x =>
            {
                language.ResourceManager.ApplyResources(x,this.Name+"."+ x.Name);
            });
            language.ResourceManager.ApplyResources(this, this.Name);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            form2 = new Form2();
            form2.Show();
        }
    }
}
